/* Copyright (c) 2013 APDM.
*
* PlotSensors.js
*
* Plot data from one or two monitors from a given JSON file
*/

// JSHint stuff
/* global utils:false, singleSensorPlot:false */

// Minimum width of plot window
var minWidth = 800;

// Minimum value for plot height
var minPlotHeight = 84;

// Size of viewport
var viewWidth;
var viewHeight;

// Padding around plots
var leftPad = 50;
var rightPad = 20;
var topPad = 10;
var bottomPad = 10;
var pageBottomPad = 20;

// The JSON data loaded from files
var monitorData;
var eventsData;

// Size of each graph
var plotWidth;
var plotHeight;

// The current monitors being plotted
var priMonitor;
var sndMonitor = null;

// Flags to determine which sensors get plotted
var plotAccel = false;
var plotGyro = false;
var plotMag = false;
var plotOrient = false;

// The list of the available sensor name
var availableSensors = [];

// List of all the plots currently being displayed.
var allPlots = [];

// The zoom behavior
var zoom;

// Position and x value under mouse at zoom
var mouseXValue;
var mouseX;

// Number of zooms
var zoomInCount = 0;

// Is script waiting for new data from the server?
var loadingNewData = false;

// Options selected
var selectedOptions = {
    removeMean: false,
    plotMagnitude: false,
    bufferPercent: 0.02,
    minRangeOrientation: 0.1,
    scale: 1,
    translate: [0,0]
};

// Redraw all the plots if the window resizes
$(window).resize(generatePlots);

// Get the paths of the files to use
var serverInfo = utils.parseURLParams(window.location.href);

// Get half the size of the plot
var halfPlotWidth = function() {
    if (plotWidth === 0) return 0;
    return plotWidth / 2;
};


function windowSize(numPlots) {
    // Take off an extra 30 for scroll bars, This may not be an issue
    // in the RawDataView. Remove the height of the options area.
    viewWidth = $(window).width() - 30;
    viewHeight = $(window).height() -
        $("#plotting_options_container").height() - 30;
    if (viewWidth < minWidth) {
        viewWidth = minWidth;
    }
    var minHeight = (minPlotHeight + topPad + bottomPad) * numPlots + pageBottomPad;
    if (viewHeight < minHeight) {
        viewHeight = minHeight;
    }
}

// Determine the size of each plot, n is number of plots
function setPlotSize(n) {
    plotWidth = viewWidth - leftPad - rightPad;
    plotHeight = ((viewHeight - pageBottomPad) / n) - pageBottomPad;
}


// Get the data from the JSON files
if (serverInfo.eventsFile) {
    d3.json(serverInfo.eventsFile, function(error, data) {
        if (error) return console.warn(error);
        eventsData = data.events;

        // Go ahead and do the date conversion here for the events
        if (eventsData) {
            eventsData.forEach( function(d) {
                d.startTime = new Date(0).setUTCMilliseconds(d.startTime/1000);
                d.endTime = new Date(0).setUTCMilliseconds(d.endTime/1000);
            });
        }
    });
}


d3.json(serverInfo.dataFile, function(error, data) {
    if (error) return console.warn(error);
    console.log("Loading data file: " + serverInfo.dataFile);
    monitorData = data;

    // Populate the dropdowns
    setMonitorSelections(monitorData.monitorIds, false);
    setMonitorSelections(monitorData.monitorIds, true);

    // Create the key
    createPlotKey();

    // Make sure the the starting monitor matches what is selected in
    // drop down.
    priMonitor = findMonitorByCaseId(monitorData.monitorIds[0].caseId);
    
    createPlottedSensorCheckboxes(priMonitor);
    createMeanMagCheckboxes();
    createEventsKey();
    newPlotGenerate();

    // Remove the loading dialog
    d3.select("#loading-dialog").remove();

});


// Create the containers for the plots - these will be updated by
// generatePlots()
function newPlotGenerate() {
    var plotSensors = setListOfAvailableSensors(priMonitor, sndMonitor);
    windowSize(plotSensors.numPlots);
    var plotSvg = d3.select("body")
        .append("div")
        .attr("id", "plot_div")
        .append("svg")
        .attr("class", "plot_svg");

    var mouseG = plotSvg.append("g")
        .attr("id", "pointer-mask")
        .attr("transform", "translate(" + leftPad + ",0)");

    mouseG.append("rect")
        .attr("id", "pointer-mask-rect");

    generatePlots();
}

// Plot the data for a given monitor
function generatePlots() {
    var plotSensors = setListOfAvailableSensors(priMonitor, sndMonitor);

    // Do nothing if there are no plots to view
    if (plotSensors.numPlots < 1) return;
    
    // Get window size
    windowSize(plotSensors.numPlots);
    setPlotSize(plotSensors.numPlots);

    var plotSvg = d3.select(".plot_svg")
        .attr("width", viewWidth)
        .attr("height", viewHeight);

    // Create the groups for the sensor plots
    var gData = plotSvg.selectAll(".plot_container")
        .data(plotSensors.priSensorList,
              function(d) { return d.name; });
    var gEnter = gData.enter();
    gData.exit().remove();

    gEnter.insert("g", "#pointer-mask").attr("class", "plot_container")
        .attr("transform",function(d,i) {return "translate(0," +
                                         (topPad + (plotHeight + bottomPad) * i) +
                                         ")";})
        .attr("id", function(d) { return d.name; });

    // Plot each of the sensors
    allPlots = [];
    for (var i = 0; i < plotSensors.numPlots; i++) {
        var selection = plotSvg.select("#" + plotSensors.priSensorList[i].name);
        var primaryData = plotSensors.priSensorList[i];
        var secondaryData = null;
        var data = { points: [], plotSecondary: false, name: null };

        if (sndMonitor !== null) {
            secondaryData = plotSensors.sndSensorList[i];
            data.plotSecondary = true;
        }

        primaryData.data.forEach( function(d, i) {
            data.points[i] = { primary:primaryData.data[i],
                        secondary: (secondaryData ? secondaryData.data[i] : null) };
        });
        data.name = primaryData.name;

        var plot = singleSensorPlot()
            .margin({ top: topPad, right: rightPad, bottom: bottomPad, left: leftPad })
            .height(plotHeight).width(plotWidth)
            .vertLabel(primaryData.name + " (" + primaryData.attributes.Units + ")")
            .showHorzTicks(false);
        
        if (i === plotSensors.numPlots - 1) {
            plot.horzLabel("Time")
                .showHorzTicks(true)
                .height(plotHeight + pageBottomPad)
                .margin({ top: topPad, right: rightPad,
                          bottom: bottomPad + pageBottomPad, left: leftPad });
        }
        setDataAccessorFunctions(data, plot);
        selection.datum([data.points]).call(plot);
        allPlots.push({selection:selection, plot:plot, data:data });
    }

    // Add a clear rect to catch all the pointer events
    zoom = d3.behavior.zoom().scaleExtent([1, 100]).on("zoom", zoomChanged);
    zoom.scale(selectedOptions.scale);
    zoom.translate([selectedOptions.translate[0], selectedOptions.translate[1]]);
    var mouseG = d3.select("#pointer-mask")
        .attr("transform", "translate(" + leftPad + ",0)");

    mouseG.select("rect")
        .attr("width", plotWidth - rightPad - leftPad)
        .attr("height", viewHeight - topPad - bottomPad)
        .attr("fill", "#FFFFFF")
        .attr("fill-opacity", "0")
        .attr("pointer-events", "all")
        .call(zoom);

    createResamplingDialog();
}

function setDataAccessorFunctions(data, plot) {
    plot.time1( function(d) {
        return (new Date(0).setUTCMilliseconds(d.primary.time/1000)); })
        .plotSecondary(false);
    var mean = { x1:0, y1:0, z1:0, scalar1:0,
                 x2:0, y2:0, z2:0, scalar2:0 };

    if (selectedOptions.removeMean) {
        mean.x1 = d3.mean(data.points.map(function(d) { return d.primary.x; }));
        mean.y1 = d3.mean(data.points.map(function(d) { return d.primary.y; }));
        mean.z1 = d3.mean(data.points.map(function(d) { return d.primary.z; }));
        if (data.name === "Orientation") {
            mean.scalar = d3.mean(
                data.points.map(function(d) { return d.primary.scalar; }));
        }
        if (data.plotSecondary) {
            mean.x2 = d3.mean(data.points.map(function(d) { return d.secondary.x; }));
            mean.y2 = d3.mean(data.points.map(function(d) { return d.secondary.y; }));
            mean.z2 = d3.mean(data.points.map(function(d) { return d.secondary.z; }));
            if (data.name === "Orientation") {
                mean.scalar2 = d3.mean(
                    data.points.map(function(d) { return d.primary.scalar; }));
            }
        }
    }

    if (selectedOptions.plotMagnitude) {
        plot.x1(function(d) {
            return (Math.sqrt(Math.pow(d.primary.x - mean.x1, 2) +
                              Math.pow(d.primary.y - mean.y1, 2) +
                              Math.pow(d.primary.z - mean.z1, 2) +
                              Math.pow((data.name === "Orientation" ?
                                        d.primary.scalar - mean.scalar1 : 0), 2)));
        }).y1(null).z1(null).scalar1(null);
        if (data.plotSecondary) {
            plot.time2( function(d) {
                return (new Date(0).setUTCMilliseconds(d.primary.time/1000)); })
                .x2(function(d) {
                    return (Math.sqrt(Math.pow(d.secondary.x - mean.x2, 2) +
                                      Math.pow(d.secondary.y - mean.y2, 2) +
                                      Math.pow(d.secondary.z - mean.z2, 2) +
                                      Math.pow((data.name === "Orientation" ?
                                                d.secondary.scalar - mean.scalar2 : 0), 2)));
                })
                .y2(null).z2(null).scalar2(null)
                .plotSecondary(true);
        }
        return;
    }

    plot.x1(function(d) { return (d.primary.x - mean.x1); })
        .y1(function(d) { return (d.primary.y - mean.y1); })
        .z1(function(d) { return (d.primary.z - mean.z1); });

    if (data.name === "Orientation") {
        plot.scalar1(function(d) { return (d.primary.scalar - mean.scalar1); });
    } else {
        plot.scalar1(null);
    }

    if (data.plotSecondary) {
        plot.time2( function(d) {
            return (new Date(0).setUTCMilliseconds(d.secondary.time/1000)); })
            .x2(function(d) { return (d.secondary.x - mean.x2) ; })
            .y2(function(d) { return (d.secondary.y - mean.y2); })
            .z2(function(d) { return (d.secondary.z - mean.z2); })
            .plotSecondary(true);
        if (data.name === "Orientation") {
            plot.scalar2(function(d) { return d.secondary.scalar - mean.scalar2; });
        } else {
            plot.scalar2(null);
        }
    }
}


// Update all the plots with the new zoom level.
function zoomChanged() {
    // Waiting to load data, don't do anything but set the event to
    // the previous zoom and scale
    if (loadingNewData) {
        return;
    }

    // Need these to determine the correct translate to keep mouse
    // under cursor
    mouseX = d3.event.sourceEvent.clientX - leftPad;
    mouseXValue = allPlots[0].plot.xScale().invert(mouseX);
    
    // Check stride to see if more points are available
    if ((monitorData.monitors[0].Calibrated[0].attributes.stride[0] > 1) &&
        (zoom.scale() > 3)) {
        zoomIn();
        return;

    } else if (zoom.scale() < 1.5 && zoomInCount > 0) {
        zoomOut();
        return;
    }

    // Handle a pan off the plot.
    if (zoomInCount > 0) {
        if (zoom.translate()[0]  > 0) {
            console.log("Pan off left edge...");
            loadingNewData = true;
            if (pan(false, -0.25)) {
                return;
            }
            loadingNewData = false;
        }
        if ((zoom.translate()[0] / (zoom.scale() - 1)) <
            -(allPlots[0].plot.xScale().range()[1])) {
            console.log("Pan off right edge...");
            loadingNewData = true;
            if (pan(true, 0.25)) {        // Add a quarter of the range
                return;
            }
            loadingNewData = false;
        }
    }
    
    allPlots.forEach( function(d) {
        d.plot.scale(zoom.scale()).translate(zoom.translate());
        d.selection.call(d.plot); });
    // Make sure that we are copying the values, not a reference
    zoom.translate(allPlots[0].plot.translate());
    selectedOptions.translate[0] = zoom.translate()[0];
    selectedOptions.translate[1] = zoom.translate()[1];
    selectedOptions.scale = allPlots[0].plot.scale();
}


function zoomIn() {
    var indexes = findRange(true, (halfPlotWidth() / 2), false);
    var length = plotWidth + halfPlotWidth();
    console.log("Getting new zoom IN data...");
    zoomInCount = zoomInCount + 1;
    showResamplingDialog(true);
    loadingNewData = true;
    sendDataRequest(indexes, length);
    selectedOptions.scale = 2;
}


function zoomOut() {
    var indexes = findRange(false, plotWidth * (2/3), false);
    var length = plotWidth + halfPlotWidth();
    console.log("Getting new zoom OUT data...");
    zoomInCount = zoomInCount - 1;
    showResamplingDialog(true);
    loadingNewData = true;
    sendDataRequest(indexes, length);
    selectedOptions.scale = 2;
}


// Get data on either the left or right off the screen
function pan(isRightPan, offset) {
    var indexes = findRange(false, offset, true);
    var length = plotWidth * 2;

    // Can't update, because we are at the end of the range
    if (!indexes) {
        return false;
    }
    showResamplingDialog(true);
    sendDataRequest(indexes, length);
    selectedOptions.scale = d3.event.scale;
    return true;
}


function sendDataRequest(indexes, length) {
    var senList = "[\"" + availableSensors.join("\",\"") + "\"]";
    var json =
        "{\"displayWidth\": " + length +
        ",\"sensors\": " + senList +
        ",\"startRow\": " + indexes.startIdx +
        ",\"endRow\": " + indexes.endIdx +
        "}";
    // Get new data from the server
    $.post("http://" + serverInfo.serverHost + ":" +
           serverInfo.serverPort + "/static/html/",
           {
               requestType: "motionstudio",
               options: json,
               dataFile: monitorData.DataFileName,
               jsonFile: monitorData.jsonFileName
           },
           function(d) { updateMonitorData(d); },
           "json");
}


function findRange(zoomIn, offset, isPan) {
    var totalLen = monitorData.monitors[0].Calibrated[0].attributes.dims[0] - 1;
    var selected = monitorData.monitors[0].Calibrated[0].attributes.selectedDims[0];
    var start = monitorData.monitors[0].Calibrated[0].attributes.startDims[0];
    var stride = monitorData.monitors[0].Calibrated[0].attributes.stride[0];
    var end = (selected * stride) + start;
    var range = (end - start);
    // This assumes a scale of 1
    var pointsPerPixel = range / (plotWidth * 2);
    var startIdx;
    var endIdx;

    if (isPan) {
        // Don't do anything if we are already at the left or right end
        // datarange
        if ((start <= 0 && offset < 0) ||
            ((end + stride) >= totalLen && offset > 0)) {
            return false;
        }

        var shift = range * offset;
        // Now we want a new start and end index for the new sample
        startIdx = start + shift;
        endIdx = end + shift;
    } else {
        // Now we want a new start and end index for the new sample
        if (zoomIn) {
            // Find the index into data that matches
            var xZero = allPlots[0].plot.xScale().invert(0);
            var i = 0;
            while (monitorData.monitors[0].Calibrated[0].data[i].time / 1000 < xZero) {
                i++;
            }
            var plotStartIdx = (i * stride) + start;

            startIdx = plotStartIdx - (offset * pointsPerPixel);
            endIdx =  startIdx + ((plotWidth + offset) * pointsPerPixel);
        } else {
            startIdx = start - Math.floor(offset * pointsPerPixel);
            endIdx = end + Math.ceil(offset * pointsPerPixel);
        }
    }

    if (startIdx <= 0) {
        // Shitf the end index
        endIdx = endIdx + Math.abs(startIdx);
        startIdx = 0;

        if (endIdx >= totalLen) {
            endIdx = totalLen;
        }
    }
    if (endIdx > totalLen) {
        // Shift the start index
        startIdx = startIdx - (endIdx - totalLen);
        endIdx = totalLen;
        if (startIdx < 0) {
            startIdx = 0;
        }
    }
    return {startIdx: Math.floor(startIdx), endIdx: Math.ceil(endIdx)};
}


// Update the monitor data
function updateMonitorData(newData) {
    console.log("Received new data");
    monitorData = newData;

    priMonitor = findMonitorByCaseId(priMonitor.caseId);
    if (sndMonitor) {
        sndMonitor = findMonitorByCaseId(sndMonitor.caseId);
    }

    allPlots.forEach( function(d) {
        var primary = findCalibratedSensorByName(priMonitor, d.data.name);
        var secondary = findCalibratedSensorByName(sndMonitor, d.data.name);
        var points = [];
        primary.data.forEach( function(d, i) {
            points[i] = { primary:primary.data[i],
                          secondary: (secondary ? secondary.data[i] : null) };
        });
        d.data.points = points;
        d.selection.datum([d.data.points]);
    });
    // Get the position of value under the mouse
    var mousePos = allPlots[0].plot.xScale()(mouseXValue);
    allPlots.forEach( function(d) {
        d.plot.scale(selectedOptions.scale)
            .translate([0,0]);
        d.selection.call(d.plot);
    });
    var newMousePos = allPlots[0].plot.xScale()(mouseXValue);
    selectedOptions.translate[0] = mousePos - newMousePos;
    allPlots.forEach( function(d) {
        // Get the new position of the value that was under the mouse,
        // and using the old position, update the transform so the
        // same value remains under the mouse.
        d.plot.translate(selectedOptions.translate);
        d.selection.call(d.plot);
    });
    zoom.translate(allPlots[0].plot.translate());
    zoom.scale(allPlots[0].plot.scale());

    // Send a fake mousemove event so the zoom behavior updates
    // everything correctly
    var e = document.createEvent('UIEvents');
    e.initUIEvent('mousemove', true, true, window, 1);
    d3.select("#pointer-mask-rect").node().dispatchEvent(e);

    // Delay to try and keep zooms from overloading
    setTimeout(function() {
        showResamplingDialog(false);
        loadingNewData = false;
    }, 500);
}


function createResamplingDialog() {
    var g = d3.select("#pointer-mask");
    g.append("rect")
        .attr("class", "dialog_resampling")
        .attr("x", ((plotWidth / 2) - 100))
        .attr("y", ((viewHeight / 2) - 50))
        .attr("width", 200).attr("height", 40)
        .attr("rx", 10).attr("ry", 10)
        .attr("opacity", 0);

    g.append("text")
        .attr("class", "dialog_resampling_text")
        .attr("x", ((plotWidth / 2) - 50))
        .attr("y", ((viewHeight / 2) - 25))
        .attr("opacity", 0)
        .text("Resampling...");
}

function showResamplingDialog(showDialog) {
    var opacity = 0;
    if (showDialog) {
        opacity = 0.90;
    }
    d3.select(".dialog_resampling")
        .attr("opacity", opacity);
    d3.select(".dialog_resampling_text")
        .attr("opacity", opacity);
}


// Handlers for changes in the monitor select dropdowns
var primarySelectChange = function () {
    console.log("Selecting " + this.value + " for plotting");
 
    priMonitor = findMonitorByCaseId(this.value);
    if (priMonitor) {
        generatePlots();
    }
};
var secondarySelectChange = function () {
    console.log("Selecting " + this.value + " for plotting");
    sndMonitor = findMonitorByCaseId(this.value);
    generatePlots();
};

// Populate the "Select Monitor" drop down with the available monitors
function setMonitorSelections(monitorIds, isSecondary) {
    var dropdownId = "#primary_monitor_select";
    var dropdownText = "Select Monitor: ";
    var monitors = monitorIds;
    var selectFunc = primarySelectChange;

    if (isSecondary) {
        // No comparison if only one monitor
        if (monitorIds.length <= 1)
            return;

        monitors = [];
        monitors.push( { monitorLabel: "None",
                     caseId: "None" });
        monitorIds.forEach( function (d) {
            monitors.push(d);
        });
        dropdownId = "#secondary_monitor_select";
        dropdownText = "Compare to: ";
        selectFunc = secondarySelectChange;
    }
    d3.select(dropdownId)
        .text(dropdownText);

    d3.select(dropdownId)
        .append("select")
        .on("change", selectFunc);

    d3.select(dropdownId)
        .select("select").selectAll("option")
        .data(monitors)
        .enter()
        .append("option")
        .attr("value", function (d) { return d.caseId; })
        .text( function (d) {
            if (d.caseId === "None") {
                return d.caseId;
            } else {
                return ("#" + d.caseId.match(/[1-9][0-9]*$/) +
                        ": " + d.monitorLabel);
            }});
}

// Add checkboxes for each available sensor to set which are plotted
function createPlottedSensorCheckboxes(monitor) {

    // Get a list of all the sensors in the "Calibrated" section.
    var sensors = [];
    monitor.Calibrated.forEach( function(d) {
        sensors.push(d.name);
    });

    if (!monitor.attributes) {
        // Assume that all the sensors are active
        availableSensors = sensors.sort();
        plotAccel = true;
        plotGyro = true;
        plotMag = true;
        plotOrient = true;
    } else {
        if (monitor.attributes.AccelerometersEnabled === "1") {
            availableSensors.push("Accelerometers");
            plotAccel = true;
        }
        if (monitor.attributes.GyroscopesEnabled === "1") {
            availableSensors.push("Gyroscopes");
            plotGyro = true;
        }
        if (monitor.attributes.MagnetometersEnabled === "1") {
            availableSensors.push("Magnetometers");
            plotMag = true;
        }
        // Since we don't have enabled flag, see if its in the list
        sensors.forEach( function(d) {
            if (d === "Orientation") {
                availableSensors.push(d);
                plotOrient = true;
            }
        });
    }

    if (availableSensors.length > 0) {
        // Create checkboxes for each available sensor
        d3.select("#sensor_selection").selectAll("input")
            .data(availableSensors)
            .enter()
            .append("label")
            .text(function (d) { return d; })
            .append("input")
            .attr("checked", true)
            .attr("type", "checkbox")
            .attr("value", function (d) { return d; })
            .on("change", onPlottedSensorCheckBoxChange);
    }
}

function onPlottedSensorCheckBoxChange() {
    var sensorName = this.value;
    if (sensorName === "Accelerometers") {
        if (this.checked) {
            plotAccel = true;
        } else {
            plotAccel = false;
        }
    } else if (sensorName === "Gyroscopes") {
        if (this.checked) {
            plotGyro = true;
        } else {
            plotGyro = false;
        }
    } else if (sensorName === "Magnetometers") {
        if (this.checked) {
            plotMag = true;
        } else {
            plotMag = false;
        }
    } else if (sensorName === "Orientation") {
        if (this.checked) {
            plotOrient = true;
        } else {
            plotOrient = false;
        }
    }
    generatePlots();
}

function createMeanMagCheckboxes() {
    // Checkbox for magnitude
    var box = d3.select("#mean_mag_selection")
        .append("label")
        .text("Remove mean")
        .append("input")
        .attr("type", "checkbox")
        .attr("value", "removeMean")
        .on("change", onMeanCheckboxChange);
    if (selectedOptions.removeMean) {
        box.attr("checked", true);
    }

    box = d3.select("#mean_mag_selection")
        .append("label")
        .text("Magnitude")
        .append("input")
        .attr("type", "checkbox")
        .attr("value", "plotMagnitude")
        .on("change", onMagnitudeChange);
    if (selectedOptions.plotMagnitude) {
        box.attr("checked", true);
    }
}

function onMeanCheckboxChange() {
    if (this.checked) {
        selectedOptions.removeMean = true;
    } else {
        selectedOptions.removeMean = false;
    }
    allPlots.forEach( function(d) {
        setDataAccessorFunctions(d.data, d.plot);
        d.selection.call(d.plot);
    });
}


function onMagnitudeChange() {
    if (this.checked) {
        selectedOptions.plotMagnitude = true;
    } else {
        selectedOptions.plotMagnitude = false;
    }
    allPlots.forEach( function(d) {
        setDataAccessorFunctions(d.data, d.plot);
        d.selection.call(d.plot);
    });
}

// Find a sensor in a monitor's calibrated group by a given name
function findCalibratedSensorByName(monitor, name) {
    if (monitor === null) {
        return null;
    }
    var sensors = monitor.Calibrated;
    for (var i = 0; i < sensors.length; i++) {
        if (sensors[i].name === name) {
            return sensors[i];
        }
    }
}


// Find a monitor in monitorData by a given Case ID
function findMonitorByCaseId(id) {
    for (var i = 0; i < monitorData.monitors.length; i++) {
        if (monitorData.monitors[i].caseId === id) {
            return monitorData.monitors[i];
        }
    }
    return null;
}

// Create the list of sensors based on monitor attributes
function setListOfAvailableSensors(monitorOne, monitorTwo) {
    var priSensorList = [];
    var sndSensorList = [];
    var numPlots = 0;

    if (plotAccel) {
        sndSensorList[numPlots] = findCalibratedSensorByName(monitorTwo, "Accelerometers");
        priSensorList[numPlots++] = findCalibratedSensorByName(monitorOne, "Accelerometers");
    }
    if (plotGyro) {
        sndSensorList[numPlots] = findCalibratedSensorByName(monitorTwo, "Gyroscopes");
        priSensorList[numPlots++] = findCalibratedSensorByName(monitorOne, "Gyroscopes");
    }
    if (plotMag) {
        sndSensorList[numPlots] = findCalibratedSensorByName(monitorTwo, "Magnetometers");
        priSensorList[numPlots++] = findCalibratedSensorByName(monitorOne, "Magnetometers");
    }
    if (plotOrient) {
        sndSensorList[numPlots] = findCalibratedSensorByName(monitorTwo, "Orientation");
        priSensorList[numPlots++] = findCalibratedSensorByName(monitorOne, "Orientation");
    }

    return { priSensorList: priSensorList,
             sndSensorList: sndSensorList,
             numPlots: numPlots };
}


// Create the key for the plot
function createPlotKey() {

    var lineLength = 20;

    var key = d3.select(".plot_key")
        .append("svg")
        .attr("width", 100)
        .attr("height", 20)
        .append("g");
    
    key.append("line")
        .attr("class", "line_x")
        .attr("x1", 0).attr("y1", 10)
        .attr("x2", lineLength).attr("y2", 10);

    key.append("text")
        .attr("text-anchor", "middle")
        .attr("x", lineLength + 5)
        .attr("y", 13)
        .text("x");

    key.append("line")
        .attr("class", "line_y")
        .attr("x1", lineLength + 14).attr("y1", 10)
        .attr("x2", (lineLength + 7) * 2).attr("y2", 10);

    key.append("text")
        .attr("text-anchor", "middle")
        .attr("x", (lineLength + 7) * 2 + 5)
        .attr("y", 13)
        .text("y");

    key.append("line")
        .attr("class", "line_z")
        .attr("x1", (lineLength + 7) * 2 + 10).attr("y1", 10)
        .attr("x2", (lineLength + 7) * 3 + 3).attr("y2", 10);

    key.append("text")
        .attr("text-anchor", "middle")
        .attr("x", (lineLength + 7) * 3 + 8)
        .attr("y", 13)
        .text("z");

}


// Create the key for plotted events
function createEventsKey() {
    if (!eventsData) return;

    var svg = d3.select(".event_key")
        .append("svg")
        .attr("height", 20)
        .attr("width", eventsData.length * 100);
    
    svg.selectAll("g")
        .data(eventsData)
        .enter()
        .append("g")
        .append("rect")
        .attr("class", function(d) { return "event_rect " + d.type; })
        .attr("x", function(d, i) { return i * 100; })
        .attr("y", 0)
        .attr("width", 20).attr("height", 20);

    svg.selectAll("g")
        .data(eventsData)
        .append("text")
        .attr("text-anchor", "start")
        .attr("x", function(d, i) { return (i * 100) + 24; })
        .attr("y", 13)
        .text(function(d) { return d.name; });
}
